@@ -1,5 +1,5 @@
 /*!
- * Copyright 2017-2018 Hitachi Vantara. All rights reserved.
+ * Copyright 2019 Hitachi Vantara. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,6 +22,7 @@
  * @module services/data.service
  * @property {String} name The name of the module.
  */
+//TODO: Remove all the unnecessary methods
 define(
     [],
     function() {
@@ -43,7 +44,7 @@ define(
        * @return {Object} The dataService api
        */
       function factory($http, $q) {
-        var baseUrl = "/cxf/browser";
+        var baseUrl = "/cxf/browser-new";
         var httpRequestCancellers = [];
           return {
           getDirectoryTree: getDirectoryTree,
@@ -106,7 +107,7 @@ define(
         }
 
         function search(path, value) {
-          return _httpGet([baseUrl, "search", encodeURIComponent(path), value].join("/"));
+          return _httpGet([baseUrl, "search", encodeURIComponent(path), encodeURIComponent(value)].join("/"));
         }
 
         /**
@@ -151,9 +152,9 @@ define(
          */
         function saveFile(path, name, fileName, override) {
           if (fileName === null) {
-            return _httpGet([baseUrl, "saveFile", encodeURIComponent(path), name, override].join("/"));
+            return _httpGet([baseUrl, "saveFile", encodeURIComponent(path), encodeURIComponent(name), override].join("/"));
           }
-          return _httpGet([baseUrl, "saveFile", encodeURIComponent(path), name, fileName, override].join("/"));
+          return _httpGet([baseUrl, "saveFile", encodeURIComponent(path), encodeURIComponent(name), encodeURIComponent(fileName), override].join("/"));
         }
 
         /**
@@ -168,10 +169,10 @@ define(
         function checkForSecurityOrDupeIssues(path, name, fileName, override) {
           if (fileName === null) {
             return _httpGet([baseUrl, "checkForSecurityOrDupeIssues",
-              encodeURIComponent(path), name, override].join("/"));
+              encodeURIComponent(path), encodeURIComponent(name), override].join("/"));
           }
           return _httpGet([baseUrl, "checkForSecurityOrDupeIssues",
-            encodeURIComponent(path), name, fileName, override].join("/"));
+            encodeURIComponent(path), encodeURIComponent(name), encodeURIComponent(fileName), override].join("/"));
         }
 
         /**
@@ -236,7 +237,7 @@ define(
          */
         function rename(id, path, newName, type, oldName) {
           return _httpPost([baseUrl, "rename", encodeURIComponent(id),
-            encodeURIComponent(path), newName, type, oldName].join("/"));
+            encodeURIComponent(path), encodeURIComponent(newName), type, oldName].join("/"));
         }
 
         /**
@@ -247,7 +248,7 @@ define(
          * @return {Promise} - a promise resolved once data is returned
          */
         function create(parent, name) {
-          return _httpPost([baseUrl, "create", encodeURIComponent(parent), name].join("/"), null);
+          return _httpPost([baseUrl, "create", encodeURIComponent(parent), encodeURIComponent(name)].join("/"), null);
         }
 
         /**
@@ -261,7 +262,7 @@ define(
          */
         function remove(id, name, path, type) {
           return _httpDelete([baseUrl, "remove",
-            encodeURIComponent(id), name, encodeURIComponent(path), type].join("/"));
+            encodeURIComponent(id), encodeURIComponent(name), encodeURIComponent(path), type].join("/"));
         }
 
         /**
