@@ -4,7 +4,7 @@
 #
 # Pentaho Data Integration
 #
-# Copyright (C) 2005-2018 by Hitachi Vantara : http://www.pentaho.com
+# Copyright (C) 2005 - 2019 by Hitachi Vantara : http://www.hitachivantara.com
 #
 # *****************************************************************************
 #
@@ -39,6 +39,9 @@ export MOZILLA_FIVE_HOME LD_LIBRARY_PATH
 # Fix for GTK Windows issues with SWT
 export GDK_NATIVE_WINDOWS=1
 
+# Fix GTK 3 issues with SWT
+export SWT_GTK3=0
+
 # Fix overlay scrollbar bug with Ubuntu 11.04
 export LIBOVERLAY_SCROLLBAR=0
 
@@ -137,10 +140,15 @@ case `uname -s` in
 
 	Linux)
 
-            HASWEBKITGTK=`ldconfig -p | grep webkitgtk-1.0`
+            if [ -f /sbin/ldconfig ]; then
+              LDCONFIG=/sbin/ldconfig
+            else
+              LDCONFIG=ldconfig
+            fi
+            HASWEBKITGTK=`$LDCONFIG -p | grep webkitgtk-1.0`
             export LIBWEBKITGTK="$HASWEBKITGTK"
             export JavaScriptCoreUseJIT=0
-            if [ -z "$HASWEBKITGTK" ]; then
+            if [ -z "$HASWEBKITGTK" ] && [ "1" != "$SKIP_WEBKITGTK_CHECK" ]; then
               echo "#######################################################################"
               echo "WARNING:  no libwebkitgtk-1.0 detected, some features will be unavailable"
               echo "    Consider installing the package with apt-get or yum."
